#!/usr/bin/perl
use strict;
use Curses;
use Curses::Widgets;  # Included to import select_colour & scankey
use Curses::Widgets::TextField;
use Curses::Widgets::ButtonSet;
use Curses::Widgets::ProgressBar;
use Curses::Widgets::TextMemo;
use Curses::Widgets::ListBox;
use Curses::Widgets::Calendar;
use Curses::Widgets::ComboBox;
use Curses::Widgets::Menu;
use Curses::Widgets::Label;
my ($mwh, $key, $i, $p);
use constant BHASH => { VALIDATE=>"Validate All", EDIT=>"Edit XML",BROWSE=>"Browse Rules",QUIT=> "Quit" };
my @binit_desc= (  BHASH->{'VALIDATE'}, BHASH->{'EDIT'} , BHASH->{'BROWSE'} ,  BHASH->{'QUIT'});
my (@widgets, $description,$title);

use XML::LibXML;
use Data::Dumper;

my @tid;

sub open_and_validate_file{
 my ($xmlfilename,$dtdfilename)=@_;
 my $parser = XML::LibXML->new();
 $parser->pedantic_parser(1);

 my $dtd ; 
 my  $doc;
  eval{ 
      $dtd =XML::LibXML::Dtd->new("",$dtdfilename); 
      $doc = $parser->parse_file( $xmlfilename );
  };
 if ($@) {  
  $title="Error!";
  $description.=  "XML Not valid.$@ \n" ;
  return undef;
 }
 return xmlvalidate($parser,$dtd,$doc);
}


sub open_and_validate_string{
my ($xmlstring,$dtdfilename)=@_;
my $dtd=XML::LibXML::Dtd->new("",$dtdfilename); 
my $parser = XML::LibXML->new();
my  $doc;
eval{
 $doc = $parser->parse_string( $xmlstring );
 };
 if ($@ ne '') { # The $@ could contain the error
  $title="Error!";
  $description.=  "XML Not valid.$@ \n" ;
  return undef;
}return xmlvalidate($parser,$dtd,$doc);
}

sub xmlvalidate{
my ($parser,$dtd,$xmldoc)=@_;
 $parser->validation(1);
 
eval{
$parser->complete_attributes(1);
$xmldoc->validate($dtd);
};
if ($@ eq '') { # The $@ could contain the error
  $title="Info";
  $description.=  "XML valid.\n" ;
  return $xmldoc;
} else {
  $title="Error!";
  $description.=  ("XML not valid 'cause of $@ !\n");
 return undef;
}
}
sub parse_pbacl{
    my ($pbaclfilename,$dtdpb)=@_;
    my $valid=undef;
    my $xmldoc=open_and_validate_file($pbaclfilename,$dtdpb);
    if(defined $xmldoc)
    {
      $valid=1;
     foreach my $cur_node ($xmldoc->findnodes('/PassBroker/Trust/File/@*'))
     {

	   if($cur_node->getName() eq "XMLFilename"){
	       my $acl=open_and_validate_file( $cur_node->getValue("XMLFilename"),"/etc/pb/dtds/PassBrokerSecret.dtd");
	       if(defined $acl)
	       {
	          $title="Info";
		  $description.= ($cur_node->getValue("XMLFilename")." is valid!");
	       }
	      }elsif($cur_node->getName() eq "XMLOwnerUId"){
	       my $uid = $cur_node->getValue("XMLOwnerUId");
	      $description.= " UID ".$uid."\n";
	    }elsif($cur_node->getName() eq "XMLOwnerGId"){
		my $uid = $cur_node->getValue("XMLOwnerGId") ;
		$description.= " GID ".$uid."\n";
	      }

     }
    }
    return $xmldoc;
}
sub create_tree_list{
  my ($doc,$xpath,$title)=@_;
  my @nodes=$doc->findnodes($xpath);
  my @values=();
  foreach my $node (@nodes)
  {
      push @values,  $node->getValue();
    
    $description.=  $_   ."\n" foreach  ($node->getValue() );
  }
  
    
   my  $lb = Curses::Widgets::ListBox->new({
       CAPTION   => 'List',
       CAPTIONCOL  => 'yellow',
       COLUMNS   => 10,
       LINES	 => 3,
       VALUE	 => 0,
       INPUTFUNC   => \&scankey,
       FOREGROUND  => 'white',
       BACKGROUND  => 'black',
       SELECTEDCOL => 'green',
       BORDER	 => 1,
       BORDERCOL   => 'red',
       FOCUSSWITCH => "\t",
       X	 => 1,
       Y	 => 1,
       TOPELEMENT  => 0,
       LISTITEMS   => [@values],
       });
        $lb->execute($mwh);

     $lb->draw($mwh, 1);
}

sub pb_xpath{
my ($xmldoc)=@_;
 foreach my $cur_node ($xmldoc->findnodes( '/PassBroker/Trust/@* | /PassBroker/Trust/File/@* | /PassBroker/Trust/File/PHPolicy//@*'))
   {
       if($cur_node->parentNode->getName eq "Trust"){
	   if($cur_node->getName() eq "tid"){
	    print $cur_node->getValue("tid")."\n";
	  }
       }elsif($cur_node->parentNode->getName() eq "File"){
       if($cur_node->getName() eq "XMLFilename"){
	    print "XMLFilename:".$cur_node->getValue("XMLFilename")."\n";
	  }elsif($cur_node->getName() eq "XMLOwnerUId"){
	    print "XMLOwnerGId:".$cur_node->getValue("XMLOwnerUId")."\n";
	  }elsif($cur_node->getName() eq "XMLOwnerGId"){
	    print "XMLOwnerGId:".$cur_node->getValue("XMLOwnerGId")."\n";
	  }
       }
	elsif($cur_node->parentNode->getName eq "FromCallerScript"){
	     if($cur_node->getName() eq "ScriptName"){
	   print "ScriptName:".$cur_node->getValue("ScriptName")."\n";
	 }elsif($cur_node->getName() eq "ScriptFunctionName"){
	   print "ScriptFunctionName:".$cur_node->getValue("ScriptFunctionName")."\n";
	 }elsif($cur_node->getName() eq "ScriptOwnerUId"){
	   print "ScriptOwnerUId:".$cur_node->getValue("ScriptOwnerUId")."\n";
	 }    
	}	
       
   }
}
$mwh = new Curses;
noecho();
halfdelay(5);
$mwh->keypad(1);
$mwh->syncok(1);
curs_set(0);
leaveok(1);

# Draw the main window, and wait for a key press (the scankey
# function is imported from the Curses::Widgets module)
main_win();
while(1){
$description='';
comment_box(<<EOF);
Validate    : Validate all XML files and print validation results
Browse Rules: Enters in interactive browsing of XML Structure
Quit        : Exits PB Administration Tool and return to Shell
EOF
my $d=initchoice();
if( $binit_desc[$d] eq BHASH->{'VALIDATE'})
{
  parse_pbacl("/etc/pb/pbacl.xml","/etc/pb/dtds/PassBroker.dtd");
  
  memo_box($title,$description);
}elsif( $binit_desc[$d] eq BHASH->{'BROWSE'} ){
  comment_box("Browse");
  my $validdoc=parse_pbacl("/etc/pb/pbacl.xml","/etc/pb/dtds/PassBroker.dtd");
  if($validdoc)
  {
   create_tree_list($validdoc,'/PassBroker/Trust/@*',"coa"); 
   
  }
   memo_box($title,$description);

  scankey($mwh);
}elsif( $binit_desc[$d] eq BHASH->{'QUIT'} ){
  last;
}
}
comment_box("Press Any Key to Exit");

$key = scankey($mwh);


sub main_win {

  $mwh->erase();

  # This function selects a few common colours for the foreground colour
  $mwh->attrset(COLOR_PAIR(select_colour(qw(red black))));
  $mwh->box(ACS_VLINE, ACS_HLINE);
  $mwh->attrset(0);

  $mwh->standout();
  $mwh->addstr(0, 1, "Welcome to the PB Administration Tool");
  $mwh->standend();
}
END {
  # The END block just ensures that Curses always cleans up behind
  # itself
  endwin();
}
sub initchoice {
my ($x,$y);
  $mwh->getmaxyx($y, $x);
my $bchoices = Curses::Widgets::ButtonSet->new({
  Y           => 2,
  X           => $x/2-12,
  FOREGROUND  => 'white',
  BACKGROUND  => 'blue',
  BORDER      => 1,
  LABELS      => [  @binit_desc ],
  LENGTH      => 20,
  HORIZONTAL  => 0,
  });
  my $key=$bchoices->execute($mwh);
  $bchoices->draw($mwh);
  return -1 if($key eq "\t");
  
  my $d= $bchoices->getField('VALUE');
  return $d;
}
sub memo_box {
  my ($title,$message) =@_;
  my ($cwh, $y, $x, @lines, $i, $line);
  my $memo;

  # Get the main screen max y & X
  $mwh->getmaxyx($y, $x);

  # Render the comment box
  $memo = Curses::Widgets::TextMemo->new({
    CAPTION     => $title,
    BORDER      => 1,
    LINES       => 5,
    COLUMNS     => $x - 4,
    Y           => $y - 8,
    X           => 1,
    VALUE       => $message,
    FOREGROUND  => 'white',
    BACKGROUND  => 'blue',
    READONLY    => 1,
    });
  $memo->execute($mwh);  
  $memo->draw($mwh);
}

 sub comment_box {
  my $message = shift;
  my ($cwh, $y, $x, @lines, $i, $line);
  my $label;

  # Get the main screen max y & X
  $mwh->getmaxyx($y, $x);

  # Render the comment box
  $label = Curses::Widgets::Label->new({
    CAPTION     => 'Help',
    BORDER      => 1,
    LINES       => 5,
    COLUMNS     => $x - 4,
    Y           => $y - 8,
    X           => 1,
    VALUE       => $message,
    FOREGROUND  => 'white',
    BACKGROUND  => 'blue',
      });
   $label->draw($mwh);
}

sub set_value_textfield{
my ($text,$wname)=@_;
my ($x,$y);
  $mwh->getmaxyx($y, $x);
 my $tf = Curses::Widgets::TextField->new({
    CAPTION     => 'Set Value for '.$wname,
    CAPTIONCOL  => 'yellow',
    COLUMNS     => 10,
    MAXLENGTH   => 255,
    MASK        => undef,
    VALUE       => $text,
    INPUTFUNC   => \&scankey,
    FOREGROUND  => undef,
    BACKGROUND  => 'black',
    BORDER      => 1,
    BORDERCOL   => 'red',
    FOCUSSWITCH => "\t\n",
    CURSORPOS   => 0,
    TEXTSTART   => 0,
    PASSWORD    => 0,
    X           => $x/2-12,,
    Y           => 3,
    READONLY    => 0,
    });

  $tf->draw($mwh, 1);
  return $tf;
}

sub set_value_area_field {
  my ($title,$message) =@_;
  my ($cwh, $y, $x, @lines, $i, $line);
  my $memo;

  # Get the main screen max y & X
  $mwh->getmaxyx($y, $x);

  # Render the comment box
  $memo = Curses::Widgets::TextMemo->new({
    CAPTION     => $title,
    BORDER      => 1,
    LINES       => 5,
    COLUMNS     => $x/2-8,
    Y           => 4,
    X           => 1,
    VALUE       => $message,
    FOREGROUND  => 'white',
    BACKGROUND  => 'blue',
    READONLY    => 0,
    });
  $memo->execute($mwh);  
  $memo->draw($mwh);
}
