/****************************************************************************
** $Id: $
**
** xml_parser.c
**
** Created : 030904
**
** Copyright (C) 2004  Stefano Di Paola All rights reserved.
**
** This file is part of the passbroker files package.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
**
** Contact stefano.dipaola@wisec.it if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <libgen.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xmlerror.h>
#include <libxml/xpathInternals.h>
#include <string.h> 
#ifdef PHP_MOD
  #include "php.h"
  #include "php_ini.h"
  #include "SAPI.h"
  #include "php_main.h"
  #include "../standard/info.h"
  #include "ext/standard/php_string.h"
  #include "php_globals.h"
  #include "Zend/zend_ini.h"
#endif

#include "passbroker.h"
#ifdef PHP_MOD
#include "php_pb.h"
#endif
#include "xml_parser.h"
#ifndef PHP_MOD
  #define DEBUG2
  #define STAND_ALONE
#endif
struct acl_nodes *head=NULL;
char *theerror;
void xmlGenericErrorDefaultFunc2	(void *ctx ATTRIBUTE_UNUSED,
				 const char *msg,
				 ...);

extern char *check_file(struct  acl_nodes *acl_node );
extern int get_gowner_id_by_name(char *gownername);
extern char *get_uowner_name_by_id(int uid);



void xmlGenericErrorDefaultFunc2(void *ctx ATTRIBUTE_UNUSED, const char *msg, ...) {
//    #ifdef DEBUG2
     va_list args;
  	if (xmlGenericErrorContext == NULL)
       xmlGenericErrorContext = (void *) stderr;
    
  	va_start(args, msg);
   #ifndef PHP_MOD
  	vfprintf((FILE *)xmlGenericErrorContext, msg, args);
  #else
     char dtderror[2048];
        extern void set_err();
  	vsnprintf(dtderror,2047, msg, args);
	theerror= estrdup(dtderror) ;
	set_err();
  #endif
 //	  zend_error(E_WARNING,msg,args );
 
 	va_end(args);
//   #endif 
 }


//a wrapper for isdigit isalnum etc. but for all the string
char is_something(char *s,int (*isfunp)(int c))
{ 
  char *p;
 p=s;
  while( isfunp(*p) &&(*p++!='\0'));
  return (*p);
}

  void set_error(char *msg)
{
#ifdef PHP_MOD
 extern void set_err();
 extern char *concat(char *src,char *dst,int dup);
 
  set_err();
  theerror= estrdup(msg) ;
#else
// theerror=estrdup(msg);
//  zend_error( E_CORE_WARNING,"%s",msg);
        fprintf(stderr,"ERRORE:%s",msg);
//    fflush(stderr);
  exit(1);
#endif
}

static int attach_Dtd(xmlDocPtr doc,char *dtdpath,char *dtdroot)
{

	 
      xmlDtdPtr dtd = xmlParseDTD(NULL,(const xmlChar *) dtdpath); /* parse the DTD */
        if(!dtd)
	{
	   set_error("sorry dtd parsing problem");
	}
	   dtd->name = strdup((xmlChar*)dtdroot); /* use the given root */
        
        doc->intSubset = dtd;
        if (doc->children == NULL)
	 xmlAddChild((xmlNodePtr)doc, (xmlNodePtr)dtd);
        else 
	 xmlAddPrevSibling(doc->children, (xmlNodePtr)dtd);
    
    xmlValidCtxt cvp;
   #ifndef DEBUG2
     cvp.userData = (void *)  stderr;
     cvp.error    = (xmlValidityErrorFunc)  xmlGenericErrorDefaultFunc2;
     cvp.warning  = (xmlValidityWarningFunc)  xmlGenericErrorDefaultFunc2;
    #else
       cvp.userData = (void *)  stderr;
       cvp.error    = (xmlValidityErrorFunc)  NULL;
       cvp.warning  = (xmlValidityWarningFunc)  NULL;
    #endif

  if (!xmlValidateDtd(&cvp,  doc, dtd))
    {
 #ifdef DEBUG2
	printf("%s\n",dtdpath);
	set_error ("XML contents does not validate   against DTD\n");
 	 return 0;
#endif
   }else{
 #ifdef DEBUG2
   		 perror ( "Validated!\n");
#endif

	}
	// xmlFree(dtd->name);
	// dtd->name=NULL;
	 return 1;
}

static xmlDocPtr getdoc (char *docname,char *dtdpath,char *dtdroot )
{
	xmlDocPtr doc;
	doc = xmlParseFile(docname);
	
	if (doc==NULL ) {
		set_error("Document not parsed successfully. \n");
		return NULL;
	}
	 if( !attach_Dtd(doc,dtdpath,dtdroot)) {
	   set_error( "Problems with DTD.\n");
	 return NULL;
       }
        	return doc;
}

static xmlDocPtr  get_aclrc(char *xmlbuf,char *dtdpath, char *dtdroot)
{
 xmlDocPtr doc;
 doc = xmlParseMemory(xmlbuf,strlen(xmlbuf)) ;
 if (doc==NULL ) {
 	   set_error("Document not parsed successfully. \n" );
 	 return NULL;
 }
       
 if(! attach_Dtd(doc,dtdpath,dtdroot)) {
	   set_error( "Problems with DTD.\n");
	 return NULL;
   }      
	return doc;
}

static xmlXPathObjectPtr getnodeset(xmlDocPtr doc, xmlChar *xpath)
{
	
	xmlXPathContextPtr context;
	xmlXPathObjectPtr result;

	context = xmlXPathNewContext(doc);
    	if(context == NULL) {
      	 	 set_error( "Error: unable to create new XPath context\n");
        	xmlFreeDoc(doc); 
       		return NULL;
    	}
	result = xmlXPathEvalExpression(xpath, context);
    
	if(result==NULL || xmlXPathNodeSetIsEmpty(result->nodesetval)){
                 set_error( "No result\n");
                xmlXPathFreeContext(context); 
       		xmlFreeDoc(doc); 
		return NULL;
	}
	
	xmlXPathFreeContext(context);
	return result;
}

static void add_acl(xmlNodePtr cur)
{
  	struct acl_nodes *newnode;
  	struct acl_nodes *appnode; 
  	newnode=(struct acl_nodes *)malloc(sizeof(struct acl_nodes));
  	newnode->node=cur;
	newnode->secretDoc=NULL;
   	newnode->ciNode= NULL;
  	newnode->tid= NULL;
	newnode->secret_filename= NULL;
 	newnode->ufownerid= -1;
  	newnode->gfownerid= -1;
  	newnode->ci= NULL;
  	newnode->next=NULL;
  	appnode=head;
  	head=newnode;
  	head->next=appnode;
}
static void free_acl(struct acl_nodes  *head)
{
   struct acl_nodes *tmp;
   struct acl_nodes *tmp2;
  tmp=head;
   while(tmp)
   {
    xmlFree(tmp->tid);
    xmlFree(tmp->secret_filename );
      free_caller_info(tmp->ci);
    tmp2=tmp->next;
    free(tmp);
    tmp=tmp2;
   }
}

static xmlChar *getencodedstring(xmlChar *st)
{
  xmlChar  *stringISO ;
 int lenUTF = strlen(st);
 int lenISO = lenUTF;
 stringISO=(xmlChar *)malloc(lenUTF*sizeof(xmlChar)+1);
 memset(stringISO,0,lenUTF*sizeof(xmlChar)+1);
 UTF8Toisolat1(stringISO, &lenISO, st, &lenUTF) ;
  return stringISO;
}

static void  parse_aclrc(char  *xmlbuf ,char *dtdpath)
{
    xmlNodePtr cur;
    int size;
    int i;
    xmlDocPtr doc;
    xmlChar *tid;
    if(head!=NULL)
    {
     free_acl(head);
     head=NULL;
    }
    //printf("DTD %s\n",dtdpath);
     doc=get_aclrc( xmlbuf, dtdpath,"PassBroker") ;
     
    if(doc==NULL){
        //set_error("sorry doc null!\n");
      head=NULL;
      return ;
    }

    xmlXPathObjectPtr result= getnodeset(  doc, "/PassBroker/Trust/@* | /PassBroker/Trust/File/@* | /PassBroker/Trust/File/PHPolicy//@*");
    if(result==NULL)
    {
      set_error("no result given!\n");
      head=NULL;
      return ;
    }
    xmlNodeSetPtr nodes=result->nodesetval;
    
    size = (nodes>0) ? nodes->nodeNr : 0;
 
 #ifdef STAND_ALONE
#ifdef DEBUG2
    fprintf(stdout, "Result (%d nodes):\n", size);
 #endif
#endif
    for(i = 0; i < size; ++i) {
	if(nodes->nodeTab[i]!=NULL)
	{
	  if(nodes->nodeTab[i]->type == XML_ELEMENT_NODE) {
		cur = nodes->nodeTab[i];   	    
			
#ifdef STAND_ALONE
	#ifdef DEBUG2
	            fprintf(stdout, "= element node \"%s\"\n", cur->name );
	#endif
#endif
		    if(cur->parent){
		     	    xmlChar *key=xmlGetProp(cur->parent,cur->name);
   			#ifdef STAND_ALONE
#ifdef DEBUG2
			    fprintf(stdout, "=parent %s = content \"%s\": cont %s\n",cur->parent->name, cur->name, key );
			#endif
#endif
			    xmlFree(key);
		    }
	    } else {
		cur = nodes->nodeTab[i];
		
		#ifdef STAND_ALONE
#ifdef DEBUG2
			fprintf(stdout, "= nodeelse \"%s\": type %d\n", cur->name, cur->type);
		#endif
#endif
		    if(cur->parent){
		            xmlChar *utfkey=xmlGetProp(cur->parent,cur->name);
			    if(utfkey){
          	              xmlChar * key=getencodedstring(utfkey);
			      xmlFree(utfkey);
			      if(!(xmlStrncmp(cur->parent->name,"Trust",MIN(strlen(cur->parent->name),5))))
			      {
			         if(!xmlStrncmp(cur->name,"tid",MIN(strlen(cur->name),3)))
				 {
				      tid=strdup( key);
				    }
			      }else if(!(xmlStrncmp(cur->parent->name,"File",MIN(strlen(cur->parent->name),4)))  )
			      {
				 if(head==NULL || (head->node)!=cur->parent)
			         { 	add_acl(cur->parent);
					head->tid=strdup(tid);
					xmlFree(tid);
				 }
				 if(!xmlStrncmp(cur->name,"XMLFilename",MIN(strlen(cur->name),11)))
				 {  
			          	head->secret_filename=strdup(key);
					
				#ifdef STAND_ALONE
					#ifdef DEBUG2
					printf("xxxxxxxxxxxxXMLFilename %s %s\n",key,head->secret_filename);
					#endif
				#endif
					
				 }else if(!xmlStrncmp(cur->name,"XMLOwnerUId",MIN(strlen(cur->name),11)))
				 {
			        	if(!is_something(key,isdigit) && head->gfownerid<0)
			        		head->ufownerid=atoi(key);
					else if(head->ufownerid<0)
						head->ufownerid=get_uowner_id_by_name(key);
			        	else{
				    		 set_error("Error something went wrong while getting uid");
					    }
				 }else if(!xmlStrncmp(cur->name,"XMLOwnerGId",MIN(strlen(cur->name),11)))
				 {
			        	if(!is_something(key,isdigit) && head->gfownerid<0)
			     			head->gfownerid=atoi(key);
					else if(head->gfownerid<0)
						head->gfownerid=get_gowner_id_by_name(key);
					else{
				    		 set_error("Error something went wrong while getting gid");	 
					    }
				 }
				 
			      }else if(!(xmlStrncmp(cur->parent->name,"FromCallerScript",MIN(strlen(cur->parent->name),16))))
			      {
			          struct caller_info *ci_tmp;
				if(head->ci==NULL || (head->ciNode)!=cur->parent)
				{
				    head->ciNode=cur->parent;
				    ci_tmp= new_caller_info( NULL,-1,-1,NULL);
				   if(head->ci==NULL)
			              head->ci=ci_tmp; 
			            else
				     head->ci= add_caller_info(head->ci,ci_tmp);
				}
				if(!xmlStrncmp(cur->name,"ScriptName",MIN(strlen(cur->name),10 )))
			       {
			        head->ci->file_name=strdup((char *)key);
				 }else if(!xmlStrncmp(cur->name,"ScriptFunctionName",MIN(strlen(cur->name),18)))
			       { 
				  head->ci->func_name =strdup((char *)key);
			     }else if(!xmlStrncmp(cur->name,"ScriptOwnerUId",MIN(strlen(cur->name),14)))
			      {
			       if(!is_something(key,isdigit) && head->ci->file_name_uid<0)
					     head->ci->file_name_uid=atoi(key);
				     else if(  head->ci->file_name_uid<0)
					     head->ci->file_name_uid=get_uowner_id_by_name(key);
				     else{
					      set_error("Error something went wrong while getting uid");
					 }
			     }else if(!xmlStrncmp(cur->name,"ScriptOwnerGId",MIN(strlen(cur->name),14)))
			      {
				 if(!is_something(key,isdigit) && head->ci->file_name_gid<0)
					     head->ci->file_name_gid=atoi(key);
				     else if( head->ci->file_name_gid<0)
					     head->ci->file_name_gid=get_uowner_id_by_name(key);
				     else{   set_error("Error something went wrong while getting uid");
					 }
			     }
 			     }
			    #ifdef STAND_ALONE
#ifdef DEBUG2
		                fprintf(stdout, "=parent %s = contentelse \"%s\": cont %s\n",cur->parent->name,  cur->name, key );
		           #endif
#endif
			      xmlFree(key);
		    }}
	    }
       }
   }
   xmlFreeDoc(doc);
}


static struct trusted_secret *parse_aclsecret(xmlDocPtr doc)
{
    xmlNodePtr cur;
    int size;
    int i;
    struct trusted_secret *ts_head=NULL;
    struct trusted_secret *ts_tmp=NULL;
    xmlNodePtr ciNode=NULL;
    xmlNodePtr infoNode=NULL;
    xmlNodePtr node=NULL;
    xmlXPathObjectPtr result= getnodeset(  doc, "/PassBrokerSecret/Secret/@*|/PassBrokerSecret/Secret//*|/PassBrokerSecret/Secret//@*");
    if(result==NULL)
    {
      set_error("no result given!\n");
    }
    xmlNodeSetPtr nodes=result->nodesetval;
    
    size = (nodes>0) ? nodes->nodeNr : 0;
 
 #ifdef STAND_ALONE
#ifdef DEBUG2
    fprintf(stdout, "Result (%d nodes):\n", size);
 #endif
#endif
    for(i = 0; i < size; ++i) {
	if(nodes->nodeTab[i]!=NULL)
	{
	  if(nodes->nodeTab[i]->type == XML_ELEMENT_NODE) {
		cur = nodes->nodeTab[i];   	    
			
     	#ifdef STAND_ALONE
#ifdef DEBUG2
	            fprintf(stdout, "= element node \"%s\"\n", cur->name );
	#endif
#endif
		    if(cur->parent){
		     	  xmlChar *key=xmlNodeGetContent(cur);
   			  if(key){
			        if(!(xmlStrncmp(cur->name,"Info",MIN(strlen(cur->parent->name),4)))  )
			      {
			         xmlChar *isokey=getencodedstring(key);
 #ifdef STAND_ALONE
#ifdef DEBUG2
			         printf("INFO %s %s\n",isokey,key); 
				#endif
#endif
			        xmlFree(isokey);
							 
			      }
			      xmlFree(key);
			    }
		    }
	    } else {
		cur = nodes->nodeTab[i];
		
		#ifdef STAND_ALONE
#ifdef DEBUG2
			fprintf(stdout, "= nodeelse \"%s\": type %d\n", cur->name, cur->type);
		#endif
#endif
		    if(cur->parent){
		            xmlChar *utfkey=xmlGetProp(cur->parent,cur->name);
			    if(utfkey){
          	              xmlChar * key=getencodedstring(utfkey);
			     if(!(xmlStrncmp(cur->parent->name,"Secret",MIN(strlen(cur->parent->name),5))))
			      {
			         if(ts_head==NULL || node!=cur->parent)
				 {
				   node=cur->parent;
				   ts_tmp = new_trusted_secret(NULL,NULL,NULL);
				   if(ts_head==NULL)
				    ts_head=ts_tmp;
				    else
				    ts_head=add_trusted_secret(ts_head,ts_tmp);
				  }
			         if(!xmlStrncmp(cur->name,"sid",MIN(strlen(cur->name),3)))
				 {
				    ts_tmp->sid=strdup(key);
				       
				 }
			      }else if(!(xmlStrncmp(cur->parent->name,"FromCallerScript",MIN(strlen(cur->parent->name),16))))
			      {
			          struct caller_info *ci_tmp;
				if(ts_tmp->ci_acl==NULL || (ciNode)!=cur->parent)
				{
				    ciNode=cur->parent;
				    ci_tmp= new_caller_info( NULL,-1,-1,NULL);
				   if(ts_tmp->ci_acl==NULL)
			              ts_tmp->ci_acl=ci_tmp; 
			            else
				     ts_tmp->ci_acl= add_caller_info(ts_tmp->ci_acl,ci_tmp);
				}
				if(!xmlStrncmp(cur->name,"ScriptName",MIN(strlen(cur->name),10 )))
			        {
			         ts_tmp->ci_acl->file_name=strdup((char *)key);
				}else if(!xmlStrncmp(cur->name,"ScriptFunctionName",MIN(strlen(cur->name),18)))
			        { 
				  ts_tmp->ci_acl->func_name =strdup((char *)key);
			     }else if(!xmlStrncmp(cur->name,"ScriptOwnerUId",MIN(strlen(cur->name),14)))
			      {
			       if(!is_something(key,isdigit) && ts_tmp->ci_acl->file_name_uid<0)
					     ts_tmp->ci_acl->file_name_uid=atoi(key);
				     else if(  ts_tmp->ci_acl->file_name_uid<0)
					     ts_tmp->ci_acl->file_name_uid=get_uowner_id_by_name(key);
				     else{
					      set_error("Error something went wrong while getting uid");
					 }
			      }else if(!xmlStrncmp(cur->name,"ScriptOwnerGId",MIN(strlen(cur->name),14)))
			      {
				 if(!is_something(key,isdigit) && ts_tmp->ci_acl->file_name_gid<0)
					     ts_tmp->ci_acl->file_name_gid=atoi(key);
				     else if( ts_tmp->ci_acl->file_name_gid<0)
					     ts_tmp->ci_acl->file_name_gid=get_uowner_id_by_name(key);
				     else{
					      set_error("Error something went wrong while getting uid");
					 }
				 }
 			     }else if(!(xmlStrncmp(cur->parent->name,"Info",MIN(strlen(cur->parent->name),4)))  )
			      {
			        struct assoc_secret *as_tmp;
				if(ts_tmp->secret_info==NULL || (infoNode)!=cur->parent)
				{
				    infoNode=cur->parent;
				     as_tmp= new_assoc_secret( NULL,NULL);
				   if(ts_tmp->secret_info==NULL)
			              ts_tmp->secret_info=as_tmp; 
			            else
				     ts_tmp->secret_info= add_assoc_secret(ts_tmp->secret_info,as_tmp);
				}
			      #ifdef STAND_ALONE
                                 #ifdef DEBUG2
			         printf("%%%%%%%%%%%%%%%%%%%%%%%%INFO %s %s\n",key,utfkey ); 
				#endif
                             #endif
				if(!xmlStrncmp(cur->name,"name",MIN(strlen(cur->name),4 )))
			        {
 #ifdef STAND_ALONE
#ifdef DEBUG2
			         printf("ASSOC name %s  \n",key);
#endif
#endif
			         //ts_tmp->ci_acl->file_name=strdup((char *)key);
				 ts_tmp->secret_info->name=strdup((char *)key);
				}else if(!xmlStrncmp(cur->name,"value",MIN(strlen(cur->name),5)))
			        { 
 #ifdef STAND_ALONE
#ifdef DEBUG2
				 printf("ASSOC Value  %s  \n",key);
#endif
#endif
				  ts_tmp->secret_info->value=strdup((char *)key);
			        }
				//ts_tmp->secret_info=strdup(key);
			         
							 
			      }
			    #ifdef STAND_ALONE
#ifdef DEBUG2
		                fprintf(stdout, "=parent %s = contentelse \"%s\": cont %s\n",cur->parent->name,  cur->name, key );
		           #endif
#endif
			      xmlFree(key);   xmlFree(utfkey);
			   
		    }
		    }
	    }
       }
   }
   xmlFreeDoc(doc);
   return ts_head;
}


static void print_nodes()
{

 struct acl_nodes *tmp_acl;
 tmp_acl=head;
 while(tmp_acl){
       printf("Inizio print_nodes \nFile=%s user=%d group=%d\n",tmp_acl->secret_filename,tmp_acl->ufownerid,tmp_acl->gfownerid);
       //tmp_acl->ci=reverse_caller_info(tmp_acl->ci);
       
       print_caller_info(tmp_acl->ci);
       printf("Fine\n");
      tmp_acl=tmp_acl->next;
  }
  
}

struct trusted_data *  get_aclsecrets(struct acl_nodes *an_head)
{
 struct acl_nodes *tmp_acl;
 struct trusted_data *td_head=NULL;
 struct trusted_data *td_tmp=NULL;
 char *dtdfile;
 #ifdef PHP_MOD
      dtdfile=strdup(DTD_SECRETFILE_PATH);
 #else
      dtdfile=strdup(DTD_SECRETFILE_PATH);
 #endif

 tmp_acl=an_head;
 
 while(tmp_acl){
     char *the_aclfilename;
     the_aclfilename= check_file( tmp_acl ); 
    
     if(the_aclfilename!=NULL)
     {

       	tmp_acl->secretDoc=getdoc( the_aclfilename ,dtdfile ,"PassBrokerSecret");
      
       	if(tmp_acl->secretDoc){
       		struct trusted_secret *ts_tmp;
       		struct trusted_secret *ts=parse_aclsecret( tmp_acl->secretDoc);
        	ts_tmp=ts;
       		while(ts_tmp)
       		{
          		ts_tmp->ci_acl=reverse_caller_info(ts_tmp->ci_acl);
          		ts_tmp=ts_tmp->next;
       		}
    		tmp_acl->ci=reverse_caller_info(tmp_acl->ci);
     		tmp_acl->ci=copy_caller_info(tmp_acl->ci );
    	  	//td_tmp=new_trusted_data(tmp_acl->tid, ts,copy_caller_info(tmp_acl->ci ) );
       		 td_tmp=new_trusted_data(tmp_acl->tid, ts,   tmp_acl->ci  );
       		if(td_head==NULL)
			td_head=td_tmp;
		else
	 		td_head=add_trusted_data(td_head,td_tmp);
        }
        free(the_aclfilename);
    }else{
	set_error("XMLFilename problem");
	return NULL;
     }
     tmp_acl=tmp_acl->next;
  }
   free(dtdfile);
   return td_head;
}



struct trusted_data *  parsexml(char *xmlb , char *rcdtd ) {
struct trusted_data *td;
    if(head!=NULL){
      free_acl(head);
      head=NULL;
    }
    /* Init libxml */     
   xmlInitParser();
   LIBXML_TEST_VERSION
    xmlSetGenericErrorFunc(xmlGenericErrorContext, (xmlGenericErrorFunc)xmlGenericErrorDefaultFunc2);
     parse_aclrc(xmlb,rcdtd);
  if(head){
   td = get_aclsecrets(head);
    }else{
   xmlCleanupParser();
   return NULL;
   }
  /* Shutdown libxml */
  xmlCleanupParser();

  return  td;
}


#ifdef STAND_ALONE
 int main(int argc, char **argv) {
    /* Parse command line and process file */
    FILE *fh;
    struct stat statb;
    struct trusted_data *td;
 char *xmlb;
    if((argc < 3) || (argc > 4)) {
	  set_error( "Error: wrong number of arguments.\n");
    } 
   
   /*Open xml file */
   lstat( argv[1] ,&statb);
   xmlb= (char *) malloc( (statb.st_size+1)*sizeof(char));
   memset(xmlb,0, (statb.st_size+1)*sizeof(char));
   fh=fopen( argv[1],"r");
   fread((char *)xmlb ,sizeof(char),statb.st_size,fh);
   fclose(fh);
   
   /* Init libxml */     
   xmlInitParser();
   LIBXML_TEST_VERSION
   // xmlSetGenericErrorFunc(xmlGenericErrorContext, (xmlGenericErrorFunc)xmlGenericErrorDefaultFunc2);
    /* Do the main job */
  parse_aclrc(xmlb,argv[2]);
    if(head){
   td=get_aclsecrets(head);
    }else{
   xmlCleanupParser();
   return 1;
   }
   print_trusted_data(td);
    print_nodes();
   /* Shutdown libxml */
    xmlCleanupParser();
   //   free_trusted_data(td);
  // free( xmlb);
  //  free_acl(head); 
 
   return 0;
}
 #endif
